#!/usr/bin/perl

package MT::Plugin::VideoEmbedding;

use strict;
use warnings;
use MT 4.0;
use base qw( MT::Plugin );

our $DISPLAY_NAME = 'Video Embedding via BBCode'; 
our $VERSION = '2.30';

our ($plugin, $PLUGIN_MODULE, $PLUGIN_KEY);
MT->add_plugin($plugin = __PACKAGE__->new({
   id          => plugin_module(),
   key         => plugin_key(),
   name        => plugin_name(),
   description => "A text filter to use BBCode-like tag to include external video resource. You can embed an youtube video with this BBCode<p><pre>[yt]eR8swuWfKMI[/yt]</pre>",
   version     => $VERSION,
   author_name => "Emiliano Bruni",
   author_link => "http://www.ebruni.it",
   plugin_link => "http://www.ebruni.it/software/os/mtos/videoembedding/",
	 doc_link		 => "http://www.ebruni.it/software/os/mtos/videoembedding/readme.htm"
}));

MT->add_text_filter(video => { label => 'Video', on_format => sub { &vcode }, });

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
    });
}

sub vcode {
	my $s = shift;
	vfullcodes($s);
}

sub vfullcodes {
    my $s = $_[0];

		$s = &youtube($s);
		$s = &google($s);
		$s = &vimeo($s);
		$s = &myspace($s);
		$s = &mediaplayer($s);
		$s = &blip($s);

    $s = vmaincodes($s);
    return $s;
}

sub vcommentcodes {
    my $s = $_[0];

    $s =~ s!<!\*amp;lt;!g;
    $s =~ s!>!\*amp;gt;!g;
    $s = vmaincodes($s);
    $s =~ s!\*amp;!&!g;

    return $s;
}
sub vmaincodes {
    my $s = $_[0];

    $s =~ s!!!g;
    $s =~ s!!!g;
    $s =~ s!!!g;
    $s =~ s!!!g;
    $s =~ s!!!g;
    $s =~ s!!!g;
    $s =~ s!!!g;

    $s = MT::Util::html_text_transform($_[0]);
    $s =~ s!(<p>.*?</p>)!vsmarty_pnts($1)!geis;
    $s =~ s!<linebreak>!\n!g;

    return $s;
}

sub vsmarty_pnts {
    my $s = $_[0];
    my $smarty = $MT::Template::Context::Global_filters{'smarty_pants'};
		return $smarty ? $smarty->($s, '1') : $s;
}

sub youtube {
	my $er	= sub() {
		my ($v,$height,$width,$hq) = (shift,shift,shift,shift);
		return <<""
			<object width="$width" height="$height">
				<param name="movie" value="http://www.youtube.com/v/$v" />
				<param name="wmode" value="transparent" />
				<embed src="http://www.youtube.com/v/$v" type="application/x-shockwave-flash" wmode="transparent" height="$height" width="$width">
			</object>

	};
	return &_def_filter(shift,'yt',$er,350,425);
}

sub google {
	my $er	= sub() {
		my ($v,$height,$width) = (shift,shift,shift,shift);
		return <<""
			<embed style="width:${width}px;height:${height}px" flashvars="" src="http://video.google.com/googleplayer.swf?docid=$v" type="application/x-shockwave-flash"> </embed>

	};
	return &_def_filter(shift,'gv',$er,326,400);
}

sub vimeo {
	my $er	= sub() {
		my ($v,$height,$width) = (shift,shift,shift,shift);
		return <<""
			<object width="$width" height="$height">
				<param name="allowfullscreen" value="true" />
				<param name="allowscriptaccess" value="always" />
				<param name="movie" value="http://www.vimeo.com/moogaloop.swf?clip_id=$v;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1" />
				<embed src="http://www.vimeo.com/moogaloop.swf?clip_id=$v&amp;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1" type="application/x-shockwave-flash" allowfullscreen="true" allowscriptaccess="always" width="$width" height="$height"></embed>
			</object>

	};
	return &_def_filter(shift,'vm',$er,225,400);
}

sub myspace {
	my $er	= sub() {
		my ($v,$height,$width) = (shift,shift,shift);
		return <<""
			<object width="${width}px" height="${height}px">
				<param name="wmode" value="transparent"/>
				<param name="allowscriptaccess" value="always"/>
				<param name="movie" value="http://lads.myspace.com/videos/vplayer.swf"/>
				<param name="flashvars" value="a=0&ap=0&y=0&m=$v&userid=-1&showmenus=0&remove=0&t=&type=video"/>
				<embed src="http://lads.myspace.com/videos/vplayer.swf" width="$width" height="$height" flashvars="a=0&ap=0&y=0&m=$v&userid=-1&showmenus=0&remove=0&t=&type=video" type="application/x-shockwave-flash" allowscriptaccess="always" />
			</object>

	};
	return &_def_filter(shift,'ms',$er,386,430);
}

sub mediaplayer {
	my $er	= sub() {
		my ($v,$height,$width) = (shift,shift,shift,shift);
		my $ctime = time();
		my $id	= "mplayer" . &sanitize_path($v) . "_$ctime";
		my $var = "so_$ctime";
		return <<""
		<script type="text/javascript" src="{mp_path}swfobject.js"></script></p>
		<div align="center" id="$id">You don't have Flash player enabled.</div>
		<script type="text/javascript">
		var $var = new SWFObject('{mp_path}mediaplayer.swf','mpl','$width','$height','8');
		$var.addParam('allowscriptaccess','always');
		$var.addParam('allowfullscreen','true');
		$var.addVariable('height','$height');
		$var.addVariable('width','$width');
		$var.addVariable('file','$v');
		$var.addVariable('displayheight','$height');
		$var.write('$id');
		</script>

	};
	my $code	= shift;
	my $ret		= &_def_filter($code,'mp',$er,512,384);
	# extract mediaplayer_path
	my $mp_path = ($code =~ /mp_path\s*=\s*"(.+)"/) ? $1 : '';
	$ret =~ s/\{mp_path\}/$mp_path/gs;
	return $ret;
}

sub blip {
	my $er	= sub() {
		my ($v,$height,$width) = (shift,shift,shift);
		return <<""
		<object type="application/x-shockwave-flash" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" width="$width" height="$height" id="blip_$v" allowfullscreen="true" allowscriptaccess="always">
		<param name="movie" value="http://blip.tv/scripts/flash/showplayer.swf?file=http://blip.tv/rss/flash/$v&autostart=false&showguidebutton=false&showsharebutton=true" />
		<param name="allowscriptaccess" value="always" />
		<param name="allowfullscreen" value="true" />
		<param name="quality" value="best" />
		<param name="wmode" value="window" />
		<embed src="http://blip.tv/scripts/flash/showplayer.swf?file=http://blip.tv/rss/flash/$v&autostart=false&showguidebutton=false&showsharebutton=true" type="application/x-shockwave-flash" width="$width" height="$height" id="blip_embed_$v" pluginspage="http://www.macromedia.com/go/getflashplayer" wmode="window" allowfullscreen="true" allowscriptaccess="always" />
		</object>

	};
	return &_def_filter(shift,'bl',$er,360,480);
}

sub _def_filter {
	my ($s,$tag,$er,$defH,$defW)		= @_;

	my $re	= '\[' . $tag. '\s*(.*?)](.*?)\[\/' . $tag. ']';

	$s =~ s{$re}{
		my $params		= $1;
		my $v					= $2;
		my $height		= ($params =~ /height\s*=\s*"?(\d+)"?/) ? $1 : $defH;
		my $width			= ($params =~ /width\s*=\s*"?(\d+)"?/) ? $1 : $defW;
		$er->($v,$height,$width);
	}gse;

	return $s;
}


sub plugin_name     { return ($DISPLAY_NAME || plugin_module()) }
sub plugin_module   {
    $PLUGIN_MODULE or ($PLUGIN_MODULE = __PACKAGE__) =~ s/^MT::Plugin:://;
    return $PLUGIN_MODULE;
}
sub plugin_key      {
    $PLUGIN_KEY or ($PLUGIN_KEY = lc(plugin_module())) =~ s/\s+//g;
    return $PLUGIN_KEY
}

sub sanitize_path {
		my $path	= shift;
		$path =~ s/[^A-Za-z0-9]/_/g;
		return $path;
}

1;

__END__

=pod

=head1 NAME

B<VideoEmbedding>  - A text filter to use BBCode-like tag to include external video.

=head1 SYNOPSIS

To include MySpace video with id: 33634079

 [ms]33634079[/ms]

To include YouTube video with id: fn3rbKd47IY

 [yt]fn3rbKd47IY[/yt]

To include Google video with id: -1297655380651175550

 [gv]-1297655380651175550[/gv]

To include Vimeo video with id: 736158

 [vm]736158[/vm]

To include you local video using JW FLV Media Player located at /my_videos/diamond_video.flv

 [mp height="480" width="320" mp_path="/plugins/"]/my_videos/diamond_video.flv[/mp]

where mp_path is the web path where swfobject.js and mediaplayer.swf are located.

To include Blip.tc video with id: 1006787

 [bl]33634079[/bl]


=head1 HOW TO INSTALL 

=over 4

=item 1. Download the zip or tar.gz file and upload the contents of the 'plugins' folder to the 'plugins' directory of your MT installation. 
As an alternative, you can install a development version of this plugin directly from its SVN location. 
Change to the 'plugins' directory and install the plug from SVN

  svn co https://svn.micso.net:666/bruni/MTOS/MTVideoEmbedding/trunk/MTVideoEmbedding/plugins/MTVideoEmbedding/ MTVideoEmbedding

=back


=head1 USAGE

=over 4

=item 1. Write a new blog entry.

=item 2. In "Format" combo, switch format to "Video"

=item 3. Append your favorite video as [yt]fn3rbKd47IY[/yt]

=item 4. Optional you can continue to edit your page in "Rich Text" mode. The "yt" tag will be substituted with embedded code.

=item 5. Save and view page.

=back


=head1 VERSION HISTORY

=over 4

=item * 2.30 (2009-01-10)

Support for JW FLV mediaplayer had problems when more that one video had shown in the same page.

In support for JW FLV mediaplayer width and height parameter were reversed.

=item * 2.20 (2008-10-06)

Add support for blip.tv video

=item * 2.10 (2008-07-15)

Add support for JW FLV mediaplayer (http://www.jeroenwijering.com/?item=JW_FLV_Player)

=item * 2.02 (2008-05-28)

Minor changes and add some documentation

=item * 2.01 (2008-05-27)

Implementing standard MT plugin interface.

=item * 2.00 (2008-05-26)

Modified by Emiliano Bruni to optimize regex code and
implementing other video hosting services (vimeo, myspace, etc.)

=item * 1.03

Modified by David Wees

=item * 1.02 (2004)

Created by Crys Clouse;

=back

=head1 AUTHOR

Emiliano Bruni <http://www.ebruni.it>

Original code: Crys Clouse <http://forums.sixapart.com/index.php?showtopic=59263>

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2008/2009 - Emiliano Bruni <http://www.ebruni.it/>

	This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  To received a copy of the GNU General Public License
  see <http://www.gnu.org/licenses/>.


=cut
